﻿using System;
using System.Collections.Generic;
using System.Text;
using PdfSharp.Pdf;
using System.Drawing;
using PdfSharp.Drawing;
using System.IO;
using System.Drawing.Drawing2D;

namespace Web2Pdf
{
    internal class ImageToPDFGenerator
    {
        #region Constants

        private readonly int PAGE_WIDTH;
        private readonly int PAGE_HEIGHT;
        private static readonly int TOP_SPACE = 25;

        #endregion


        #region Fields

        private string title;
        private string author;
        private string subject;
        private string keywords;

        #endregion


        #region Constructors

        public ImageToPDFGenerator(int pageWidth, int pageHeight)
            : this(pageWidth, pageHeight, string.Empty, string.Empty, string.Empty, string.Empty)
        {
        }

        public ImageToPDFGenerator(int pageWidth, int pageHeight, string title, string author, string subject, string keywords)
        {
            this.PAGE_WIDTH = pageWidth;
            this.PAGE_HEIGHT = pageHeight;
            this.title = title;
            this.author = author;
            this.subject = subject;
            this.keywords = keywords;
        }

        #endregion


        #region Methods

        public byte[] Generate(Bitmap img)
        {
            if (img.Width > this.PAGE_WIDTH)
            {
                img = ResizeImage(img, this.PAGE_WIDTH);
            }

            PdfDocument document = new PdfDocument();
            document.Info.Title = this.title;
            document.Info.Author = this.author;
            document.Info.Subject = this.subject;
            document.Info.Keywords = this.keywords;
            
            // Calculate the count of the pages, into wich the image should be split.
            int pagesCount = img.Height / PAGE_HEIGHT;
            if (img.Height % PAGE_HEIGHT != 0)
            {
                pagesCount++;
            }

            // Generate the PDF pages.
            double pictureWidth;
            double pictureHeight;
            for (int i = 0; i < pagesCount; i++)
            {
                PdfPage page = document.AddPage();
                XGraphics gfx = XGraphics.FromPdfPage(page);
                Bitmap nextImagePart = img.Clone(new Rectangle(0, i * PAGE_HEIGHT, img.Width, 
                    Math.Min(PAGE_HEIGHT, img.Height - i * PAGE_HEIGHT)), img.PixelFormat);

                XImage image = XImage.FromGdiPlusImage(nextImagePart);                 
                pictureWidth = image.PixelWidth * 72 / image.HorizontalResolution;
                pictureHeight = image.PixelHeight * 72 / image.HorizontalResolution;
                gfx.DrawImage(image, (page.Width - pictureWidth) / 2, TOP_SPACE);
            }

            MemoryStream stream = new MemoryStream();
            document.Save(stream);
            byte[] pdfData = stream.ToArray();
            return pdfData;
        }

        private static Bitmap ResizeImage(Bitmap imgToResize, int destWidth)
        {
            int destHeight = (int)(imgToResize.Height * ((double)destWidth / imgToResize.Width));
            Bitmap result = new Bitmap(destWidth, destHeight);

            using (Graphics g = Graphics.FromImage((Image)result))
            {
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                g.DrawImage(imgToResize, 0, 0, destWidth, destHeight);
            }

            return result;
        }

        #endregion
    }
}
